@@ -88,10 +88,6 @@ module Nokogiri
           assert_instance_of(Nokogiri::HTML4::DocumentFragment, fragment)
         end
 
-        def test_many_fragments
-          100.times { Nokogiri::HTML4::DocumentFragment.new(html) }
-        end
-
         def test_html_fragment
           fragment = Nokogiri::HTML4.fragment("<div>a</div>")
           assert_equal("<div>a</div>", fragment.to_s)
@@ -100,11 +96,7 @@ module Nokogiri
         def test_html_fragment_has_outer_text
           doc = "a<div>b</div>c"
           fragment = Nokogiri::HTML4::Document.new.fragment(doc)
-          if Nokogiri.uses_libxml?("<= 2.6.16")
-            assert_equal("a<div>b</div><p>c</p>", fragment.to_s)
-          else
-            assert_equal("a<div>b</div>c", fragment.to_s)
-          end
+          assert_equal("a<div>b</div>c", fragment.to_s)
         end
 
         def test_html_fragment_case_insensitivity
@@ -162,13 +154,7 @@ module Nokogiri
         def test_to_xhtml
           doc = "<span>foo<br></span><span>bar</span><p></p>"
           fragment = Nokogiri::HTML4::Document.new.fragment(doc)
-          if Nokogiri.jruby? || Nokogiri.uses_libxml?(">= 2.7.0")
-            assert_equal("<span>foo<br /></span><span>bar</span><p></p>", fragment.to_xhtml)
-          else
-            # FIXME: why are we doing this ? this violates the spec,
-            # see http://www.w3.org/TR/xhtml1/#C_2
-            assert_equal("<span>foo<br></span><span>bar</span><p></p>", fragment.to_xhtml)
-          end
+          assert_equal("<span>foo<br /></span><span>bar</span><p></p>", fragment.to_xhtml)
         end
 
         def test_to_xml
@@ -180,28 +166,51 @@ module Nokogiri
         def test_fragment_script_tag_with_cdata
           doc = Nokogiri::HTML4::Document.new
           fragment = doc.fragment("<script>var foo = 'bar';</script>")
-          assert_equal("<script>var foo = 'bar';</script>",
-            fragment.to_s)
+          assert_equal(
+            "<script>var foo = 'bar';</script>",
+            fragment.to_s,
+          )
         end
 
         def test_fragment_with_comment
           doc = Nokogiri::HTML4::Document.new
           fragment = doc.fragment("<p>hello<!-- your ad here --></p>")
-          assert_equal("<p>hello<!-- your ad here --></p>",
-            fragment.to_s)
+          assert_equal(
+            "<p>hello<!-- your ad here --></p>",
+            fragment.to_s,
+          )
         end
 
         def test_element_children_counts
-          if Nokogiri.uses_libxml?("<= 2.9.1")
-            skip("#elements doesn't work in 2.9.1, see 1793a5a for history")
-          end
           doc = Nokogiri::HTML4::DocumentFragment.parse("   <div>  </div>\n   ")
           assert_equal(1, doc.element_children.count)
         end
 
         def test_malformed_fragment_is_corrected
           fragment = Nokogiri::HTML4::DocumentFragment.parse("<div </div>")
-          assert_equal("<div></div>", fragment.to_s)
+
+          if Nokogiri.uses_libxml?(">= 2.14.0")
+            assert_pattern do
+              fragment => [
+                { name: "div", attributes: [
+                    { name: "<", value: ""},
+                    { name: "div", value: ""},
+                  ]}
+              ]
+            end
+          else
+            assert_equal("<div></div>", fragment.to_s)
+          end
+        end
+
+        def test_malformed_html5_fragment_serializes_like_gumbo
+          skip_unless_libxml2(">= 2.14.0")
+
+          fragment = Nokogiri::HTML4::DocumentFragment.parse("<div </div>")
+
+          pending "libxml2 does not serialize HTML5 like gumbo (yet)" do
+            assert_equal('<div <="" div=""></div>', fragment.to_s)
+          end
         end
 
         def test_unclosed_script_tag
@@ -211,31 +220,32 @@ module Nokogiri
         end
 
         def test_error_propagation_on_fragment_parse
-          frag = Nokogiri::HTML4::DocumentFragment.parse("<hello>oh, hello there.</hello>")
-          assert(frag.errors.any? { |err| err.to_s.include?("Tag hello invalid") }, "errors should be copied to the fragment")
+          frag = Nokogiri::HTML4::DocumentFragment.parse("<hello>oh, hello there</goodbye>")
+          refute_empty(frag.errors)
         end
 
         def test_error_propagation_on_fragment_parse_in_node_context
           doc = Nokogiri::HTML4::Document.parse("<html><body><div></div></body></html>")
           context_node = doc.at_css("div")
-          frag = Nokogiri::HTML4::DocumentFragment.new(doc, "<hello>oh, hello there.</hello>", context_node)
-          assert(frag.errors.any? do |err|
-                   err.to_s.include?("Tag hello invalid")
-                 end, "errors should be on the context node's document")
+          frag = Nokogiri::HTML4::DocumentFragment.new(doc, "<hello>oh, hello there</goodbye>", context_node)
+          refute_empty(frag.errors)
         end
 
         def test_error_propagation_on_fragment_parse_in_node_context_should_not_include_preexisting_errors
-          doc = Nokogiri::HTML4::Document.parse("<html><body><div></div><jimmy></jimmy></body></html>")
-          assert(doc.errors.any? { |err| err.to_s.include?("jimmy") }, "assert on setup")
+          doc = Nokogiri::HTML4::Document.parse("<html><body><div></jimmy></body></html>")
+          refute_empty(doc.errors)
+          doc_errors = doc.errors.map(&:to_s)
 
           context_node = doc.at_css("div")
-          frag = Nokogiri::HTML4::DocumentFragment.new(doc, "<hello>oh, hello there.</hello>", context_node)
-          assert(frag.errors.any? do |err|
-                   err.to_s.include?("Tag hello invalid")
-                 end, "errors should be on the context node's document")
-          assert(frag.errors.none? do |err|
-                   err.to_s.include?("jimmy")
-                 end, "errors should not include pre-existing document errors")
+          frag = Nokogiri::HTML4::DocumentFragment.new(doc, "<hello>oh, hello there.</goodbye>", context_node)
+          refute_empty(frag.errors)
+
+          assert(
+            frag.errors.none? do |err|
+              doc_errors.include?(err.to_s)
+            end,
+            "errors should not include pre-existing document errors",
+          )
         end
 
         def test_capturing_nonparse_errors_during_fragment_clone
@@ -249,14 +259,15 @@ module Nokogiri
 
         def test_capturing_nonparse_errors_during_node_copy_between_fragments
           # Errors should be emitted while parsing only, and should not change when moving nodes.
-          frag1 = Nokogiri::HTML4.fragment("<diva id='unique'>one</diva>")
-          frag2 = Nokogiri::HTML4.fragment("<dive id='unique'>two</dive>")
+          frag1 = Nokogiri::HTML4.fragment("<div id='unique'>one</foo1>")
+          frag2 = Nokogiri::HTML4.fragment("<div id='unique'>two</foo2>")
           node1 = frag1.at_css("#unique")
           node2 = frag2.at_css("#unique")
           original_errors1 = frag1.errors.dup
           original_errors2 = frag2.errors.dup
-          assert(original_errors1.any? { |e| e.to_s.include?("Tag diva invalid") }, "it should complain about the tag name")
-          assert(original_errors2.any? { |e| e.to_s.include?("Tag dive invalid") }, "it should complain about the tag name")
+
+          refute_empty(original_errors1)
+          refute_empty(original_errors2)
 
           node1.add_child(node2)
 
@@ -271,6 +282,15 @@ module Nokogiri
           assert_instance_of(Nokogiri::HTML4::DocumentFragment, duplicate)
         end
 
+        def test_parse_with_io
+          fragment = Nokogiri::HTML4::DocumentFragment.parse(StringIO.new("<div>hello</div>"), "UTF-8")
+          assert_instance_of(HTML4::DocumentFragment, fragment)
+          assert_equal("<div>hello</div>", fragment.to_s)
+
+          fragment = Nokogiri::HTML4::DocumentFragment.parse(StringIO.new("<div>hello</div>"))
+          assert_equal("<div>hello</div>", fragment.to_s)
+        end
+
         describe "encoding" do
           describe "#fragment" do
             it "parses an encoded string" do
@@ -306,6 +326,14 @@ module Nokogiri
               assert_equal("hello world", fragment.content)
             end
 
+            it "returns a string matching an encoding passed with kwargs" do
+              input = "<div>hello world</div>"
+
+              fragment = Nokogiri::HTML4::DocumentFragment.parse(input, encoding: "ISO-8859-1")
+              assert_equal("ISO-8859-1", fragment.document.encoding)
+              assert_equal("hello world", fragment.content)
+            end
+
             it "respects encoding for empty strings" do
               fragment = Nokogiri::HTML::DocumentFragment.parse("", "UTF-8")
               assert_equal "UTF-8", fragment.to_html.encoding.to_s
@@ -336,106 +364,115 @@ module Nokogiri
             Nokogiri::XML::ParseOptions.new(Nokogiri::XML::ParseOptions::DEFAULT_HTML).norecover
           end
 
-          let(:input) { "<div>foo</div" }
+          let(:html4_huge) do
+            Nokogiri::XML::ParseOptions.new(Nokogiri::XML::ParseOptions::DEFAULT_HTML).huge
+          end
+
+          let(:input) { "<div>foo</div>" }
 
           it "sets the test up correctly" do
+            refute_predicate(html4_default, :strict?)
+            refute_predicate(html4_default, :huge?)
             assert_predicate(html4_strict, :strict?)
+            assert_predicate(html4_huge, :huge?)
           end
 
           describe "HTML4.fragment" do
-            it "has sane defaults" do
+            it "has reasonable defaults" do
               frag = Nokogiri::HTML4.fragment(input)
+
               assert_equal("<div>foo</div>", frag.to_html)
-              refute_empty(frag.errors)
+              assert_equal(html4_default, frag.parse_options)
             end
 
             it "accepts options" do
-              frag = Nokogiri::HTML4.fragment(input, nil, html4_default)
-              assert_equal("<div>foo</div>", frag.to_html)
-              refute_empty(frag.errors)
+              frag = Nokogiri::HTML4.fragment(input, nil, html4_huge)
 
-              assert_raises(Nokogiri::SyntaxError) do
-                Nokogiri::HTML4.fragment(input, nil, html4_strict)
-              end
+              assert_equal("<div>foo</div>", frag.to_html)
+              assert_equal(html4_huge, frag.parse_options)
+            end
+
+            it "accepts options as kwargs" do
+              frag = Nokogiri::HTML4::DocumentFragment.parse(input, options: html4_huge)
+
+              assert_equal("<div>foo</div>", frag.to_html)
+              assert_equal(html4_huge, frag.parse_options)
             end
 
             it "takes a config block" do
               default_config = nil
-              Nokogiri::HTML4.fragment(input) do |config|
-                default_config = config
+              frag = Nokogiri::HTML4.fragment(input) do |config|
+                default_config = config.dup
+                config.huge
               end
-              refute_predicate(default_config, :strict?)
 
-              assert_raises(Nokogiri::SyntaxError) do
-                Nokogiri::HTML4.fragment(input, &:norecover)
-              end
+              assert_equal(html4_default, default_config)
+              refute_predicate(default_config, :huge?)
+              assert_predicate(frag.parse_options, :huge?)
             end
           end
 
           describe "HTML4::DocumentFragment.parse" do
-            it "has sane defaults" do
+            it "has reasonable defaults" do
               frag = Nokogiri::HTML4::DocumentFragment.parse(input)
+
               assert_equal("<div>foo</div>", frag.to_html)
-              refute_empty(frag.errors)
+              assert_equal(html4_default, frag.parse_options)
             end
 
             it "accepts options" do
-              frag = Nokogiri::HTML4::DocumentFragment.parse(input, nil, html4_default)
-              assert_equal("<div>foo</div>", frag.to_html)
-              refute_empty(frag.errors)
+              frag = Nokogiri::HTML4::DocumentFragment.parse(input, nil, html4_huge)
 
-              assert_raises(Nokogiri::SyntaxError) do
-                Nokogiri::HTML4::DocumentFragment.parse(input, nil, html4_strict)
-              end
+              assert_equal("<div>foo</div>", frag.to_html)
+              assert_equal(html4_huge, frag.parse_options)
             end
 
             it "takes a config block" do
               default_config = nil
-              Nokogiri::HTML4::DocumentFragment.parse(input) do |config|
-                default_config = config
+              frag = Nokogiri::HTML4::DocumentFragment.parse(input) do |config|
+                default_config = config.dup
+                config.huge
               end
-              refute_predicate(default_config, :strict?)
 
-              assert_raises(Nokogiri::SyntaxError) do
-                Nokogiri::HTML4::DocumentFragment.parse(input, &:norecover)
-              end
+              assert_equal(html4_default, default_config)
+              refute_predicate(default_config, :huge?)
+              assert_predicate(frag.parse_options, :huge?)
             end
           end
 
           describe "HTML4::DocumentFragment.new" do
             describe "without a context node" do
-              it "has sane defaults" do
+              it "has reasonable defaults" do
                 frag = Nokogiri::HTML4::DocumentFragment.new(Nokogiri::HTML4::Document.new, input)
+
                 assert_equal("<div>foo</div>", frag.to_html)
-                refute_empty(frag.errors)
+                assert_equal(html4_default, frag.parse_options)
               end
 
               it "accepts options" do
-                frag = Nokogiri::HTML4::DocumentFragment.new(Nokogiri::HTML4::Document.new, input, nil, html4_default)
-                assert_equal("<div>foo</div>", frag.to_html)
-                refute_empty(frag.errors)
+                frag = Nokogiri::HTML4::DocumentFragment.new(Nokogiri::HTML4::Document.new, input, nil, html4_huge)
 
-                assert_raises(Nokogiri::SyntaxError) do
-                  Nokogiri::HTML4::DocumentFragment.new(Nokogiri::HTML4::Document.new, input, nil, html4_strict)
-                end
+                assert_equal("<div>foo</div>", frag.to_html)
+                assert_equal(html4_huge, frag.parse_options)
               end
 
               it "takes a config block" do
                 default_config = nil
-                Nokogiri::HTML4::DocumentFragment.new(Nokogiri::HTML4::Document.new, input) do |config|
-                  default_config = config
+                frag = Nokogiri::HTML4::DocumentFragment.new(Nokogiri::HTML4::Document.new, input) do |config|
+                  default_config = config.dup
+                  config.huge
                 end
-                refute_predicate(default_config, :strict?)
 
-                assert_raises(Nokogiri::SyntaxError) do
-                  Nokogiri::HTML4::DocumentFragment.new(Nokogiri::HTML4::Document.new, input, &:norecover)
-                end
+                assert_equal(html4_default, default_config)
+                refute_predicate(default_config, :huge?)
+                assert_predicate(frag.parse_options, :huge?)
               end
             end
 
             describe "with a context node" do
               let(:document) { Nokogiri::HTML4::Document.parse("<context></context>") }
               let(:context_node) { document.at_css("context") }
+              let(:input) { "<div>foo</div" }
 
               it "has sane defaults" do
                 frag = Nokogiri::HTML4::DocumentFragment.new(document, input, context_node)
@@ -473,9 +510,9 @@ module Nokogiri
             Class.new(Nokogiri::HTML4::DocumentFragment) do
               attr_accessor :initialized_with, :initialized_count
 
-              def initialize(*args)
+              def initialize(*args, **kwargs)
                 super
-                @initialized_with = args
+                @initialized_with = [args, kwargs]
                 @initialized_count ||= 0
                 @initialized_count += 1
               end
@@ -494,8 +531,11 @@ module Nokogiri
             end
 
             it "passes args to #initialize" do
-              fragment = klass.new(html, "<div>a</div>")
-              assert_equal([html, "<div>a</div>"], fragment.initialized_with)
+              fragment = klass.new(html, "<div>a</div>", options: 1)
+              assert_equal(
+                [[html, "<div>a</div>"], { options: 1 }],
+                fragment.initialized_with,
+              )
             end
           end
 
